#include "ogl.H"
#include "CFlatDisplay.H"
#include "XmdvTool.H"
#include "colormap.H"
#include "magnify.H"
#include "brush_oper.H"


CFlatDisplay::CFlatDisplay(CFlatDisplayInformation*  flat_info)
{
	this->flat_info = flat_info;
    show_dimcluster_band = false;
    show_two_axes = false;
    show_wide_axes = false;
}


CFlatDisplay::~CFlatDisplay()
{
}


//draw the canvas
void CFlatDisplay::Draw()
{
    if (!dataset) return;

    //set canvas background, clear canvas
	PrepareCanvas();

	if ( TestDimensionality() ) //some display requires minimum dimensionality
	{
		//set parameters used for drawing
		SetParameters();

		//draw brushes
		SetBrushesColorAndStyle();
		DrawBrushes();

    
		//draw unbrushed data;
		SetUnbrushedColorAndStyle();
		DrawUnbrushedRegion();
		
		//draw brushed data;
		SetBrushedColorAndStyle();
		DrawBrushedRegion();
    
		//draw the average of the brush if required.
		SetAverageColorAndStyle();
		DrawAverage();
		
		//draw axes
		DrawAxes();
    
        //draw other things such as lines links star glyphs...
//        DrawOthers();

		//free space used for drawing
		FreeSpace();
	}
}


//set parameters used for the drawing
//rewrite when necessary
void CFlatDisplay::SetParameters()
{
    dims = dataset->dims;
    data.SetLen(dims);

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == MEANBANDS )
        show_dimcluster_band = true;
    else
        show_dimcluster_band = false;

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == TWOAXES )
        show_two_axes = true;
    else
        show_two_axes = false;

    if ( dim_reducer != NULL && dim_reducer->status == APPLY && dim_reduction_info->disagree_present == WIDEAXES )
        show_wide_axes = true;
    else
        show_wide_axes = false;

    if ( show_dimcluster_band || show_two_axes )
    {
        max.SetLen(dims);
        min.SetLen(dims);
    }
}


//draw axes, or grid
//rewrite when necessary
void CFlatDisplay::DrawAxes()
{
}


//set color and style for brushes
//rewrite when necessary
void CFlatDisplay::SetBrushesColorAndStyle()
{
}


//draw brushes
void CFlatDisplay::DrawBrushes()
{
    int brnum, i, j;
    
    // draw all brushes
    for (brnum=0; brnum<MAXBRUSH; brnum++) {
        Brush *brush = &brushes[brnum];

        // if this brush is not being displayed, skip it
        if (brush->display == false) continue;
        
        DrawSingleBrush(brush);
    }

    // draw brushes in operation. The first operation drawn last
    for (i=num_oper-1; i>=0; i--)
    {
        for (j = 0; j < brush_ops[i].num_brush; j++ )
        {
            brnum = brush_ops[i].brush[j];
            Brush *brush = &brushes[brnum];

            // if this brush is not being displayed, skip it
            if (brush->display == false) continue;
    
            DrawSingleBrush(brush);
        }
    }

}


//set color and style for unbrushed data
//rewrite when necessary
void CFlatDisplay::SetUnbrushedColorAndStyle()
{
    RGBt c(interface_color[DATA]);
    glColor3f((float) c.R, (float) c.G, (float) c.B );
    canvas->SetForeground(interface_color[DATA]);
}


//draw unbrushed data
void CFlatDisplay::DrawUnbrushedRegion()
{
    //if MAGNIGY, should not draw unbrushed region.
    if ( MAGNIFY ) 
        return;
	
    int i;
    for (i=0; i<dataset->data_size; i++)
    {
        // First time through only paint regular data
        //
        if (dataset->mark[i] == okcdata::DISABLED) continue;
        if (dataset->mark[i] != okcdata::REGULAR) continue;

        if ( show_dimcluster_band || show_two_axes )
        {
            dataset->GetDataWithDimClusterExtends(data, max, min, i);
        }
        else
        {
            dataset->GetData(data, i);
        }
        if ( show_dimcluster_band )
            DrawSingleDataWithDimClusterExtends(data, max, min, i);
        else if ( show_two_axes )
            DrawSingleDataWithTwoAxes(data, max, min, i, true);
        else
            DrawSingleData(data, i);
    }
}


//set color and style for brushed data
//rewrite when necessary
void CFlatDisplay::SetBrushedColorAndStyle()
{
}


//draw brushed data
void CFlatDisplay::DrawBrushedRegion()
{
    int i;
    for (i=0; i<dataset->data_size; i++)
    {
        if (dataset->mark[i] == okcdata::DISABLED) continue;
        if (dataset->mark[i] == okcdata::REGULAR) continue;

        glColor3f((float) dataset->color[i].R, (float) dataset->color[i].G, (float) dataset->color[i].B );
        canvas->SetForeground(dataset->color[i]);
        if ( show_dimcluster_band || show_two_axes )
        {
            dataset->GetDataWithDimClusterExtends(data, max, min, i);
        }
        else
        {
            dataset->GetData(data, i);
        }
        if ( show_dimcluster_band )
            DrawSingleDataWithDimClusterExtends(data, max, min, i);
        else if ( show_two_axes )
            DrawSingleDataWithTwoAxes(data, max, min, i, false);
        else
            DrawSingleData(data, i);
    }
}


//set color and style for average of brushes
//rewrite when necessary
void CFlatDisplay::SetAverageColorAndStyle()
{
    canvas->SetForeground(interface_color[TEXT]);
}


//draw average of brushes if required
void CFlatDisplay::DrawAverage()
{
    for (int op=0; op<num_oper; op++)
    {
        // Check if this operation has OPER_AVERAGE set */
        if (!(brush_ops[op].oper & AVERAGE)) continue;

        ARR<double> average(dataset->dims);
        if (BrushOperCompAverage(op, average)==0) continue;

        glLineWidth(3.0);
        canvas->SetForeground(interface_color[TEXT]);
        DrawSingleData(average);
    }
}


//free space used for drawing
//rewrite when necessary
void CFlatDisplay::FreeSpace()
{
}

/*
//map all_data to data, then draw data. The distortion in data space is done here.
void CFlatDisplay::DrawSingleData1(ARR<double> all_data)
{
    MapData(all_data);
    DrawSingleData();
}


//map all_data to data, The distortion in data space is done here.
void CFlatDisplay::MapData(ARR<double> all_data)
{
    dataset->FlatMapData(all_data, data);
}

*/

//draw a single data. Data to be draw is stored ARR data, dimension is dims
//need to rewrite
void CFlatDisplay::DrawSingleData(ARR<double> data, int datano)
{
}


void CFlatDisplay::DrawSingleDataWithDimClusterExtends(ARR<double> mean, ARR<double> max, ARR<double> min, int datano)
{
}


void CFlatDisplay::DrawSingleDataWithTwoAxes(ARR<double> mean, ARR<double> max, 
                                             ARR<double> min, int datano, bool unbrushed)
{
}


//draw a single brush
//need to rewrite
void CFlatDisplay::DrawSingleBrush(Brush *brush)
{
}

//draw other things such as lines links star glyphs
//need to rewrite
void CFlatDisplay::DrawOthers()
{
}


void CFlatDisplay::InputKeyPress(int key)
{
    appCanvasInputKeyPress(canvas, key);
}


void CFlatDisplay::InputKeyRelease(int key)
{
    appCanvasInputKeyRelease(canvas, key);
}

